@@ -7,12 +7,13 @@ importing sub-modules.
 
 import contextlib
 from pathlib import Path
+from urllib.parse import urljoin
 
 import cssselect2
 import html5lib
 import tinycss2
 
-VERSION = __version__ = '55.0'
+VERSION = __version__ = '56.0'
 
 __all__ = [
     'HTML', 'CSS', 'Attachment', 'Document', 'Page', 'default_url_fetcher',
@@ -27,6 +28,20 @@ from .logger import LOGGER, PROGRESS_LOGGER  # noqa isort:skip
 # to work around circular imports.
 
 
+def _find_base_url(html_document, fallback_base_url):
+    """Return the base URL for the document.
+
+    See http://www.w3.org/TR/html5/urls.html#document-base-url
+
+    """
+    first_base_element = next(iter(html_document.iter('base')), None)
+    if first_base_element is not None:
+        href = first_base_element.get('href', '').strip()
+        if href:
+            return urljoin(fallback_base_url, href)
+    return fallback_base_url
+
+
 class HTML:
     """HTML document parsed by html5lib.
 
@@ -84,7 +99,7 @@ class HTML:
                     source, override_encoding=encoding,
                     transport_encoding=protocol_encoding,
                     namespaceHTMLElements=False)
-        self.base_url = find_base_url(result, base_url)
+        self.base_url = _find_base_url(result, base_url)
         self.url_fetcher = url_fetcher
         self.media_type = media_type
         self.wrapper_element = cssselect2.ElementWrapper.from_html_root(
@@ -132,7 +147,8 @@ class HTML:
     def write_pdf(self, target=None, stylesheets=None, zoom=1,
                   attachments=None, finisher=None, presentational_hints=False,
                   optimize_size=('fonts',), font_config=None,
-                  counter_style=None, image_cache=None):
+                  counter_style=None, image_cache=None, identifier=None,
+                  variant=None, version=None, custom_metadata=False):
         """Render the document to a PDF file.
 
         This is a shortcut for calling :meth:`render`, then
@@ -167,6 +183,11 @@ class HTML:
         :type counter_style: :class:`css.counters.CounterStyle`
         :param counter_style: A dictionary storing ``@counter-style`` rules.
         :param dict image_cache: A dictionary used to cache images.
+        :param bytes identifier: A bytestring used as PDF file identifier.
+        :param str variant: A PDF variant name.
+        :param str version: A PDF version number.
+        :param bool custom_metadata: A boolean defining whether custom HTML
+            metadata should be stored in the generated PDF.
         :returns:
             The PDF as :obj:`bytes` if ``target`` is not provided or
             :obj:`None`, otherwise :obj:`None` (the PDF is written to
@@ -175,10 +196,11 @@ class HTML:
         """
         return (
             self.render(
-                stylesheets, presentational_hints=presentational_hints,
-                optimize_size=optimize_size, font_config=font_config,
-                counter_style=counter_style, image_cache=image_cache)
-            .write_pdf(target, zoom, attachments, finisher))
+                stylesheets, presentational_hints, optimize_size, font_config,
+                counter_style, image_cache)
+            .write_pdf(
+                target, zoom, attachments, finisher, identifier, variant,
+                version, custom_metadata))
 
 
 class CSS:
@@ -221,10 +243,9 @@ class CSS:
         self.base_url = base_url
         self.matcher = matcher or cssselect2.Matcher()
         self.page_rules = [] if page_rules is None else page_rules
-        self.fonts = []
         preprocess_stylesheet(
             media_type, base_url, stylesheet, url_fetcher, self.matcher,
-            self.page_rules, self.fonts, font_config, counter_style)
+            self.page_rules, font_config, counter_style)
 
 
 class Attachment:
@@ -314,6 +335,5 @@ def _select_source(guess=None, filename=None, url=None, file_obj=None,
 # Work around circular imports.
 from .css import preprocess_stylesheet  # noqa isort:skip
 from .html import (  # noqa isort:skip
-    HTML5_UA_COUNTER_STYLE, HTML5_UA_STYLESHEET, HTML5_PH_STYLESHEET,
-    find_base_url)
+    HTML5_UA_COUNTER_STYLE, HTML5_UA_STYLESHEET, HTML5_PH_STYLESHEET)
 from .document import Document, Page  # noqa isort:skip
